@@ -1,13 +1,16 @@
 package Test::Needs;
 use strict;
 use warnings;
-our $VERSION = '0.001001';
+our $VERSION = '0.002000';
 $VERSION =~ tr/_//d;
 
 BEGIN {
   *_WORK_AROUND_HINT_LEAKAGE
     = "$]" < 5.011 && !("$]" >= 5.009004 && "$]" < 5.010001)
     ? sub(){1} : sub(){0};
+  *_WORK_AROUND_BROKEN_MODULE_STATE
+    = "$]" < 5.009
+    ? sub(){1} : sub(){0};
 }
 
 sub _try_require {
@@ -22,6 +25,8 @@ sub _try_require {
       or $err = $@;
   }
   if (defined $err) {
+    delete $INC{$file}
+      if _WORK_AROUND_BROKEN_MODULE_STATE;
     die $err
       unless $err =~ /\ACan't locate \Q$file\E/;
     return !1;
@@ -33,6 +38,28 @@ sub _find_missing {
   my @bad = map {
     my ($module, $version) = @$_;
     if ($module eq 'perl') {
+      $version
+        = !$version ? 0
+        : $version =~ /^[0-9]+\.[0-9]+$/ ? sprintf('%.6f', $version)
+        : $version =~ /^v?([0-9]+(?:\.[0-9]+)+)$/ ? do {
+          my @p = split /\./, $1;
+          push @p, 0
+            until @p >= 3;
+          sprintf '%d.%03d%03d', @p;
+        }
+        : $version =~ /^\x05..?$/s ? do {
+          my @p = map ord, split //, $version;
+          push @p, 0
+            until @p >= 3;
+          sprintf '%d.%03d%03d', @p;
+        }
+        : do {
+          use warnings FATAL => 'numeric';
+          no warnings 'void';
+          eval { 0 + $version; 1 } ? $version
+            : die sprintf qq{version "%s" for perl does not look like a number at %s line %s.\n},
+              $version, (caller( 1 + ($Test::Builder::Level||0) ))[1,2];
+        };
       if ("$]" < $version) {
         "perl $version (have $])";
       }
@@ -40,6 +67,11 @@ sub _find_missing {
         ();
       }
     }
+    elsif ($module =~ /^\d|[^\w:]|[^:]:[^:]|^:|:$/) {
+      die sprintf qq{"%s" does not look like a module name at %s line %s.\n},
+        $module, (caller( 1 + ($Test::Builder::Level||0) ))[1,2];
+      die
+    }
     elsif (_try_require($module)) {
       local $@;
       if (defined $version && !eval { $module->VERSION($version); 1 }) {
@@ -54,18 +86,13 @@ sub _find_missing {
     }
   }
   map {
-    if (ref) {
+    if (ref eq 'HASH') {
       my $arg = $_;
       map [ $_ => $arg->{$_} ], sort keys %$arg;
     }
-    elsif (/^[0-9]+\.[0-9]+$/) {
-      [ perl => sprintf '%.6f', $_ ];
-    }
-    elsif (/^v?([0-9]+(?:\.[0-9]+)+)$/) {
-      my @p = split /\./, $1;
-      push @p, 0
-        until @p >= 3;
-      [ perl => sprintf '%d.%03d%03d', @p ];
+    elsif (ref eq 'ARRAY') {
+      my $arg = $_;
+      map [ @{$arg}[$_*2,$_*2+1] ], 0 .. int($#$arg / 2);
     }
     else {
       [ $_ => undef ];
@@ -124,12 +151,16 @@ sub _fail_or_skip {
   }
   elsif ($INC{'Test/Builder.pm'}) {
     my $tb = Test::Builder->new;
+    my $has_plan = $tb->can('has_plan') ? 'has_plan'
+      : sub { $_[0]->expected_tests || eval { $_[0]->current_test($_[0]->current_test); 'no_plan' } };
     if ($fail) {
+      $tb->plan(tests => 1)
+        unless $tb->$has_plan;
       $tb->ok(0, "Test::Needs modules available");
       $tb->diag($message);
     }
     else {
-      my $plan = $tb->has_plan;
+      my $plan = $tb->$has_plan;
       my $tests = $tb->current_test;
       if ($plan || $tests) {
         my $skips
@@ -176,10 +207,12 @@ Test::Needs - Skip tests when modules not available
 
   use Test::Needs 'Some::Module';
 
+  # check module version
   use Test::Needs {
     'Some::Module' => '1.005',
   };
 
+  # check later
   use Test::Needs;
   test_needs 'Some::Module';
 
@@ -189,7 +222,8 @@ Test::Needs - Skip tests when modules not available
     test_needs 'Some::Module';  # skips remainder of subtest
   };
 
-  use Test::Needs 5.020;
+  # check perl version
+  use Test::Needs { perl => 5.020 };
 
 =head1 DESCRIPTION
 
@@ -204,8 +238,6 @@ If the C<RELEASE_TESTING> environment variable is set, the tests will fail
 rather than skip.  Subtests will be aborted, but the test script will continue
 running after that point.
 
-If a bare version number is specified, it is checked against the perl version.
-
 =head1 EXPORTS
 
 =head2 test_needs
